{% if report.bank_table %}
<table id="table_interests" class="table">
    <thead>
    <tr class="tabletitle">
	<th align="center">{{ report.interests.0.1 }}</th>
	<th align="center">{{ report.interests.0.2 }}</th>
    </tr>
    </thead>
    <tbody>
{% for item in report.interests %}
    {% if forloop.first %}
    {% else %}   
        <tr{% if item.0 %} class="tabletotal" {% endif %}>
            <td>{% if item.0 %}<b>{{ item.1 }}</b>{% else %}<a href="skg://Skrooge_calculator_plugin/?currentPage=0&account={{ item.1|encode }}&year={{ report.interests.0.1|encode }}">{{ item.1 }}</a>{% endif %}</td>
            <td align="right">{% if item.0 %}<b>{% endif %}{{ item.2|money|safe }}{% if item.0 %}</b>{% endif %}</td>
        </tr>    
    {% endif %}      
{% endfor %}     
    </tbody>
</table>   
{% endif %}   
