{% if report.bank_table %}
<table id="table_alarms" class="table">
    <tbody>
{% for item in report.alarms %} 
	<tr>
	    <td>{{ item.0 }}</td>
	</tr>      
{% endfor %}     
    </tbody>
</table>   
{% endif %}   
