<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_monthly" version="2">
  <MenuBar>
    <Menu name="edit"><text>Edit</text></Menu>
  </MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>  
</kpartgui>
