#!/bin/sh
EXE=skgtestimportiif

#initialisation
. `dirname $0`/init.sh

${EXE}
rc=$?
if [ $rc != 0 ] ; then
	exit $rc
fi

#checkDiff ${OUT}/skgtestimportqif1/ref.qif ${REF}/skgtestimportqif1/ref.qif 

exit 0