#!/bin/sh
EXE=skgtestimportmny3

#initialisation
. `dirname $0`/init.sh

chmod u-r ${IN}/skgtestimportmny1/notreadable.mny

${EXE}
rc=$?
if [ $rc != 0 ] ; then
	exit $rc
fi

chmod u+r ${IN}/skgtestimportmny1/notreadable.mny

exit 0