#!/bin/sh
EXE=skgtestimportcsv

#initialisation
. `dirname $0`/init.sh

${EXE}
rc=$?
if [ $rc != 0 ] ; then
	exit $rc
fi

checkDiff ${OUT}/skgtestimportcsv/bankperfect.csv ${REF}/skgtestimportcsv/bankperfect.csv 

exit 0