<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_properties" version="11" >
<MenuBar>
  <Menu name="edit"><text>Edit</text>
	  <Action name="add_property" group="edit_group_copy"/>
	  <Separator group="edit_group_copy"/>
  </Menu>
  <Menu name="view">
  	<Menu name="panels"><text>Panels</text>
		<Action name="view_properties"/>
	</Menu>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
