<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_print" version="6">
<MenuBar>
  <Menu name="file"><text>File</text>
      <Action name="file_print" />
      <Action name="file_print_preview" />
      <Separator/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
